"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Debug = require("debug");
const debug = Debug("blockchain:signature");
const crypto = require("crypto");
const Address_1 = require("./Address");
const Address_2 = require("./Address");
const Address_3 = require("./Address");
const Address_4 = require("./Address");
const SHA256 = "sha256";
const HEX = "hex";
class Signature {
    constructor(config) {
        this.config = config;
    }
    transactionToString(transaction) {
        return `${transaction.sender}:${transaction.recipient}:${transaction.amount}:` +
            `${transaction.payload || ""}:${transaction.timestamp || ""}`;
    }
    createSignature(content, privateKey) {
        const signer = crypto.createSign(SHA256);
        signer.update(content);
        signer.end();
        const key = `${Address_3.PRIVATE_SOF}${privateKey}${Address_4.PRIVATE_EOF}`;
        const signature = signer.sign(key);
        const signatureAsHex = signature.toString(HEX);
        return signatureAsHex;
    }
    verifySignature(content, signatureAsHex, publicKey) {
        const signature = Buffer.from(signatureAsHex, HEX);
        const verifier = crypto.createVerify(SHA256);
        verifier.update(content);
        verifier.end();
        const key = `${Address_1.PUBLIC_SOF}${publicKey}${Address_2.PUBLIC_EOF}`;
        const verified = verifier.verify(key, signature);
        if (verified) {
            debug("signature is valid");
        }
        else {
            debug("signature is invalid.");
        }
        return verified;
    }
    createTransactionSignature(transaction, privateKey) {
        return this.createSignature(this.transactionToString(transaction), privateKey);
    }
    verifyTransactionSignature(transaction) {
        if (!transaction || typeof transaction.signature === "undefined") {
            return false;
        }
        // transaction for a mined block
        if (transaction.sender === this.config.blockchain.mintAddress && !transaction.signature) {
            return true;
        }
        return this.verifySignature(this.transactionToString(transaction), transaction.signature, transaction.sender);
    }
}
exports.default = Signature;
//# sourceMappingURL=Signature.js.map